@@ -1,46 +1,58 @@
 /*
-    SPDX-FileCopyrightText: 2013 Marco Martin <mart@kde.org>
-
-    SPDX-License-Identifier: LGPL-2.0-or-later
-*/
+ *   Copyright 2013 Marco Martin <mart@kde.org>
+ *
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU Library General Public License as
+ *   published by the Free Software Foundation; either version 2, or
+ *   (at your option) any later version.
+ *
+ *   This program is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details
+ *
+ *   You should have received a copy of the GNU Library General Public
+ *   License along with this program; if not, write to the
+ *   Free Software Foundation, Inc.,
+ *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ */
 
+#include "private/configcategory_p.h"
 #include "configview.h"
+#include "configmodel.h"
 #include "Plasma/Applet"
 #include "Plasma/Containment"
-#include "appletcontext_p.h"
-#include "appletquickitem.h"
-#include "configmodel.h"
-#include "private/configcategory_p.h"
+//#include "plasmoid/wallpaperinterface.h"
+#include "kdeclarative/configpropertymap.h"
 
 #include <QDebug>
 #include <QDir>
 #include <QQmlComponent>
-#include <QQmlContext>
 #include <QQmlEngine>
-#include <QQmlFileSelector>
+#include <QQmlContext>
 #include <QQuickItem>
 
-#include <KAuthorized>
-#include <KLocalizedContext>
-#include <KLocalizedString>
-#include <KPackage/Package>
+#include <klocalizedstring.h>
+#include <kdeclarative/kdeclarative.h>
+#include <packageurlinterceptor.h>
+#include <KQuickAddons/ConfigModule>
 
 #include <Plasma/Corona>
 #include <Plasma/PluginLoader>
-#include <qqmlengine.h>
 
-// Unfortunately QWINDOWSIZE_MAX is not exported
-#define DIALOGSIZE_MAX ((1 << 24) - 1)
+//Unfortunately QWINDOWSIZE_MAX is not exported
+#define DIALOGSIZE_MAX ((1<<24)-1)
 
 namespace PlasmaQuick
 {
+
 //////////////////////////////ConfigView
 
 class ConfigViewPrivate
 {
 public:
     ConfigViewPrivate(Plasma::Applet *appl, ConfigView *view);
-    ~ConfigViewPrivate() = default;
+    ~ConfigViewPrivate();
 
     void init();
 
@@ -51,24 +63,20 @@ public:
     void mainItemLoaded();
 
     ConfigView *q;
-    QPointer<Plasma::Applet> applet;
+    QWeakPointer <Plasma::Applet> applet;
     ConfigModel *configModel;
     ConfigModel *kcmConfigModel;
     Plasma::Corona *corona;
-    AppletContext *rootContext;
-    QQmlEngine *engine = nullptr;
-    QQuickItem *rootItem = nullptr;
 
-    // Attached Layout property of mainItem, if any
-    QPointer<QObject> mainItemLayout;
+    //Attached Layout property of mainItem, if any
+    QWeakPointer <QObject> mainItemLayout;
 };
 
 ConfigViewPrivate::ConfigViewPrivate(Plasma::Applet *appl, ConfigView *view)
-    : q(view)
-    , applet(appl)
-    , corona(nullptr)
+    : q(view),
+      applet(appl),
+      corona(0)
 {
-    engine = new QQmlEngine(q);
 }
 
 void ConfigViewPrivate::init()
@@ -82,25 +90,25 @@ void ConfigViewPrivate::init()
         return;
     }
 
-    rootContext = new AppletContext(q->engine(), applet, nullptr);
-    rootContext->setParent(q->engine());
-
     applet.data()->setUserConfiguring(true);
 
-    KLocalizedContext *localizedContextObject = new KLocalizedContext(q->engine());
+    KDeclarative::KDeclarative kdeclarative;
+    kdeclarative.setDeclarativeEngine(q->engine());
     const QString rootPath = applet.data()->pluginMetaData().value(QStringLiteral("X-Plasma-RootPath"));
     if (!rootPath.isEmpty()) {
-        localizedContextObject->setTranslationDomain(QStringLiteral("plasma_applet_") + rootPath);
+        kdeclarative.setTranslationDomain("plasma_applet_" + rootPath);
     } else {
-        localizedContextObject->setTranslationDomain(QStringLiteral("plasma_applet_") + applet.data()->pluginMetaData().pluginId());
+        kdeclarative.setTranslationDomain("plasma_applet_" + applet.data()->pluginMetaData().pluginId());
     }
-    rootContext->setContextObject(localizedContextObject);
+    kdeclarative.setupBindings();
+    qmlRegisterType<ConfigModel>("org.kde.plasma.configuration", 2, 0, "ConfigModel");
+    qmlRegisterType<ConfigCategory>("org.kde.plasma.configuration", 2, 0, "ConfigCategory");
 
-    // FIXME: problem on nvidia, all windows should be transparent or won't show
+    //FIXME: problem on nvidia, all windows should be transparent or won't show
     q->setColor(Qt::transparent);
     q->setTitle(i18n("%1 Settings", applet.data()->title()));
 
-    // systray case
+    //systray case
     if (!applet.data()->containment()->corona()) {
         Plasma::Applet *a = qobject_cast<Plasma::Applet *>(applet.data()->containment()->parent());
         if (a) {
@@ -116,66 +124,55 @@ void ConfigViewPrivate::init()
         return;
     }
 
-    const auto pkg = corona->kPackage();
-    if (pkg.isValid()) {
-        new QQmlFileSelector(q->engine(), q->engine());
+    if (corona->kPackage().isValid()) {
+        PackageUrlInterceptor *interceptor = new PackageUrlInterceptor(q->engine(), corona->package());
+        interceptor->addAllowedPath(applet.data()->package().path());
+        q->engine()->setUrlInterceptor(interceptor);
     }
 
-    if (!qEnvironmentVariableIntValue("PLASMA_NO_CONTEXTPROPERTIES")) {
-        rootContext->setContextProperties({QQmlContext::PropertyPair{QStringLiteral("plasmoid"), QVariant::fromValue(applet.data())},
-                                           QQmlContext::PropertyPair{QStringLiteral("configDialog"), QVariant::fromValue(q)}});
-    }
+    q->setResizeMode(QQuickView::SizeViewToRootObject);
 
-    // config model local of the applet
-    QQmlComponent component(q->engine(), applet.data()->kPackage().fileUrl("configmodel"));
-    QObject *object = component.create(rootContext);
+    //config model local of the applet
+    QQmlComponent *component = new QQmlComponent(q->engine(), QUrl::fromLocalFile(applet.data()->kPackage().filePath("configmodel")), q);
+    QObject *object = component->beginCreate(q->engine()->rootContext());
     configModel = qobject_cast<ConfigModel *>(object);
 
     if (configModel) {
         configModel->setApplet(applet.data());
-        configModel->setParent(q);
     } else {
         delete object;
     }
 
-    QStringList kcms = applet.data()->pluginMetaData().value(QStringLiteral("X-Plasma-ConfigPlugins"), QStringList());
-
-    // filter out non-authorized KCMs
-    // KAuthorized expects KCMs with .desktop suffix, so we can't just pass everything
-    // to KAuthorized::authorizeControlModules verbatim
-    kcms.erase(std::remove_if(kcms.begin(),
-                              kcms.end(),
-                              [](const QString &kcm) {
-                                  return !KAuthorized::authorizeControlModule(kcm + QLatin1String(".desktop"));
-                              }),
-               kcms.end());
-
+    const QStringList kcms = KPluginMetaData::readStringList(applet.data()->pluginMetaData().rawData(), QStringLiteral("X-Plasma-ConfigPlugins"));
     if (!kcms.isEmpty()) {
         if (!configModel) {
             configModel = new ConfigModel(q);
         }
 
-        for (const QString &kcm : std::as_const(kcms)) {
-            // Only look for KCMs in the "kcms_" folder where new QML KCMs live
-            // because we don't support loading QWidgets KCMs
-            KPluginMetaData md(QLatin1String("kcms/") + kcm);
+        foreach (const QString &kcm, kcms) {
+            KPluginLoader loader(KPluginLoader::findPlugin(QLatin1String("kcms/") + kcm));
+            KPluginMetaData md(loader.fileName());
 
             if (!md.isValid()) {
-                qWarning() << "Could not find" << kcm
-                           << "requested by X-Plasma-ConfigPlugins. Ensure that it exists, is a QML KCM, and lives in the 'kcms/' subdirectory.";
+                qWarning() << "Could not find" << kcm << "specified in X-Plasma-ConfigPlugins";
                 continue;
             }
 
-            configModel->appendCategory(md.iconName(), md.name(), QString(), QLatin1String("kcms/") + kcm);
+            configModel->appendCategory(md.iconName(), md.name(), QString(), loader.fileName());
         }
     }
+
+    q->engine()->rootContext()->setContextProperty(QStringLiteral("plasmoid"), applet.data()->property("_plasma_graphicObject").value<QObject *>());
+    q->engine()->rootContext()->setContextProperty(QStringLiteral("configDialog"), q);
+    component->completeCreate();
+    delete component;
 }
 
 void ConfigViewPrivate::updateMinimumWidth()
 {
     if (mainItemLayout) {
         q->setMinimumWidth(mainItemLayout.data()->property("minimumWidth").toInt());
-        // Sometimes setMinimumWidth doesn't actually resize: Qt bug?
+        //Sometimes setMinimumWidth doesn't actually resize: Qt bug?
 
         q->setWidth(qMax(q->width(), q->minimumWidth()));
     } else {
@@ -187,7 +184,7 @@ void ConfigViewPrivate::updateMinimumHeight()
 {
     if (mainItemLayout) {
         q->setMinimumHeight(mainItemLayout.data()->property("minimumHeight").toInt());
-        // Sometimes setMinimumHeight doesn't actually resize: Qt bug?
+        //Sometimes setMinimumHeight doesn't actually resize: Qt bug?
 
         q->setHeight(qMax(q->height(), q->minimumHeight()));
     } else {
@@ -229,23 +226,23 @@ void ConfigViewPrivate::mainItemLoaded()
 {
     if (applet) {
         KConfigGroup cg = applet.data()->config();
-        cg = KConfigGroup(&cg, QStringLiteral("ConfigDialog"));
+        cg = KConfigGroup(&cg, "ConfigDialog");
         q->resize(cg.readEntry("DialogWidth", q->width()), cg.readEntry("DialogHeight", q->height()));
     }
 
-    // Extract the representation's Layout, if any
-    QObject *layout = nullptr;
+    //Extract the representation's Layout, if any
+    QObject *layout = 0;
 
-    // Search a child that has the needed Layout properties
-    // HACK: here we are not type safe, but is the only way to access to a pointer of Layout
-    const auto children = rootItem->children();
-    for (QObject *child : children) {
-        // find for the needed property of Layout: minimum/maximum/preferred sizes and fillWidth/fillHeight
-        if (child->property("minimumWidth").isValid() && child->property("minimumHeight").isValid() && child->property("preferredWidth").isValid()
-            && child->property("preferredHeight").isValid() && child->property("maximumWidth").isValid() && child->property("maximumHeight").isValid()
-            && child->property("fillWidth").isValid() && child->property("fillHeight").isValid()) {
+    //Search a child that has the needed Layout properties
+    //HACK: here we are not type safe, but is the only way to access to a pointer of Layout
+    foreach (QObject *child, q->rootObject()->children()) {
+        //find for the needed property of Layout: minimum/maximum/preferred sizes and fillWidth/fillHeight
+        if (child->property("minimumWidth").isValid() && child->property("minimumHeight").isValid() &&
+                child->property("preferredWidth").isValid() && child->property("preferredHeight").isValid() &&
+                child->property("maximumWidth").isValid() && child->property("maximumHeight").isValid() &&
+                child->property("fillWidth").isValid() && child->property("fillHeight").isValid()
+            ) {
             layout = child;
-            break;
         }
     }
     mainItemLayout = layout;
@@ -263,16 +260,21 @@ void ConfigViewPrivate::mainItemLoaded()
     }
 }
 
+
 ConfigView::ConfigView(Plasma::Applet *applet, QWindow *parent)
-    : QQuickWindow(parent)
-    , d(new ConfigViewPrivate(applet, this))
+    : QQuickView(parent),
+      d(new ConfigViewPrivate(applet, this))
 {
     setIcon(QIcon::fromTheme(QStringLiteral("configure")));
-    // Only register types once
-    [[maybe_unused]] static int configModelRegisterResult = qmlRegisterType<ConfigModel>("org.kde.plasma.configuration", 2, 0, "ConfigModel");
-    [[maybe_unused]] static int configCategoryRegisterResult = qmlRegisterType<ConfigCategory>("org.kde.plasma.configuration", 2, 0, "ConfigCategory");
     d->init();
+    qmlRegisterType<ConfigModel>("org.kde.plasma.configuration", 2, 0, "ConfigModel");
+    qmlRegisterType<ConfigCategory>("org.kde.plasma.configuration", 2, 0, "ConfigCategory");
     connect(applet, &QObject::destroyed, this, &ConfigView::close);
+    connect(this, &QQuickView::statusChanged, [=](QQuickView::Status status) {
+        if (status == QQuickView::Ready) {
+            d->mainItemLoaded();
+        }
+    });
 }
 
 ConfigView::~ConfigView()
@@ -283,56 +285,11 @@ ConfigView::~ConfigView()
             d->applet.data()->containment()->corona()->requestConfigSync();
         }
     }
-    delete d->rootItem;
-}
-
-QQmlEngine *ConfigView::engine()
-{
-    return d->engine;
-}
-
-QQmlContext *ConfigView::rootContext()
-{
-    return d->rootContext;
-}
-
-void ConfigView::setSource(const QUrl &src)
-{
-    QQmlComponent uiComponent(engine(), src);
-    if (uiComponent.isError()) {
-        for (const auto &error : uiComponent.errors()) {
-            qWarning() << error;
-        }
-    }
-    QObject *object = uiComponent.createWithInitialProperties({{QStringLiteral("parent"), QVariant::fromValue(contentItem())}}, d->rootContext);
-
-    d->rootItem = qobject_cast<QQuickItem *>(object);
-    if (!d->rootItem) {
-        return;
-    }
-    d->mainItemLoaded();
-
-    if (d->rootItem->implicitHeight() > 0 || d->rootItem->implicitWidth() > 0) {
-        resize(QSize(d->rootItem->implicitWidth(), d->rootItem->implicitHeight()));
-    }
-    d->rootItem->setSize(QSizeF(width(), height()));
-
-    connect(d->rootItem, &QQuickItem::implicitWidthChanged, this, [this]() {
-        setWidth(d->rootItem->implicitWidth());
-    });
-    connect(d->rootItem, &QQuickItem::implicitHeightChanged, this, [this]() {
-        setWidth(d->rootItem->implicitHeight());
-    });
-}
-
-QQuickItem *ConfigView::rootObject()
-{
-    return d->rootItem;
 }
 
 void ConfigView::init()
 {
-    setSource(d->corona->kPackage().fileUrl("appletconfigurationui"));
+    setSource(QUrl::fromLocalFile(d->corona->kPackage().filePath("appletconfigurationui")));
 }
 
 Plasma::Applet *ConfigView::applet()
@@ -361,10 +318,10 @@ void ConfigView::setAppletGlobalShortcut(const QString &shortcut)
     }
 
     d->applet.data()->setGlobalShortcut(shortcut);
-    Q_EMIT appletGlobalShortcutChanged();
+    emit appletGlobalShortcutChanged();
 }
 
-// To emulate Qt::WA_DeleteOnClose that QWindow doesn't have
+//To emulate Qt::WA_DeleteOnClose that QWindow doesn't have
 void ConfigView::hideEvent(QHideEvent *ev)
 {
     QQuickWindow::hideEvent(ev);
@@ -373,15 +330,14 @@ void ConfigView::hideEvent(QHideEvent *ev)
 
 void ConfigView::resizeEvent(QResizeEvent *re)
 {
-    if (!d->rootItem) {
+    if (!rootObject()) {
         return;
     }
-
-    d->rootItem->setSize(re->size());
+    rootObject()->setSize(re->size());
 
     if (d->applet) {
         KConfigGroup cg = d->applet.data()->config();
-        cg = KConfigGroup(&cg, QStringLiteral("ConfigDialog"));
+        cg = KConfigGroup(&cg, "ConfigDialog");
         cg.writeEntry("DialogWidth", re->size().width());
         cg.writeEntry("DialogHeight", re->size().height());
     }
